@@ -8,6 +8,7 @@ import json
 import logging
 import os.path
 import base64
+import traceback
 from collections import Callable
 from signal import SIGINT, SIGTERM
 from six import string_types
@@ -17,14 +18,16 @@ from circuits.core.handlers import handler
 from requests.utils import DEFAULT_CA_BUNDLE_PATH
 import resilient
 from resilient import ensure_unicode
+from resilient_lib import IntegrationError
 import resilient_circuits.actions_test_component as actions_test_component
 from resilient_circuits.decorators import *  # for back-compatibility, these were previously declared here
 from resilient_circuits.rest_helper import get_resilient_client, reset_resilient_client
 from resilient_circuits.action_message import ActionMessageBase, ActionMessage, \
-    FunctionMessage, StatusMessage, FunctionResult, BaseFunctionError
+    FunctionMessage, InboundMessage, StatusMessage, FunctionResult, BaseFunctionError
 from resilient_circuits.stomp_component import StompClient
 from resilient_circuits.stomp_events import *
 from resilient_circuits import helpers
+from resilient_circuits import constants
 
 LOG = logging.getLogger(__name__)
 
@@ -71,6 +74,7 @@ class FunctionWorker(Worker):
         try:
             yield result.get()
         except Exception as e:
+            LOG.error(traceback.format_exc())
             yield ExceptionWrapper(e)
 
 
@@ -294,6 +298,7 @@ class Actions(ResilientComponent):
         _retry_timer.register(self)
 
         # Make a worker thread-pool that will run functions
+        LOG.debug("num_workers set to %s", opts.get("num_workers"))
         self._functionworker = FunctionWorker(process=False, channel="functionworker", workers=opts.get("num_workers"))
         self._functionworker.register(self.root)
 
@@ -386,7 +391,7 @@ class Actions(ResilientComponent):
                         headers.get("message"), message))
 
     @handler("Message")
-    def on_stomp_message(self, event, headers, message):
+    def on_stomp_message(self, event, headers, message, queue):
         """STOMP produced a message."""
         # Find the queue name from the subscription id (stomp_listener_xxx)
         msg_id = event.frame.headers.get("message-id")
@@ -430,7 +435,16 @@ class Actions(ResilientComponent):
 
                 message = json.loads(mstr)
                 # Construct a Circuits event with the message, and fire it on the channel
-                if message.get("function"):
+                if queue and queue[0] == constants.INBOUND_MSG_DEST_PREFIX:
+                    channel = u"{0}.{1}".format(constants.INBOUND_MSG_DEST_PREFIX, queue[2])
+                    event = InboundMessage(source=self,
+                                           queue=queue,
+                                           headers=headers,
+                                           message=message,
+                                           frame=event.frame,
+                                           log_dir=self.logging_directory)
+
+                elif message.get("function"):
                     channel = "functions." + message["function"]["name"]
                     event = FunctionMessage(source=self,
                                             headers=headers,
@@ -602,6 +616,20 @@ class Actions(ResilientComponent):
                 queue_name = component._functions[func_name]["destination_handle"]
                 LOG.info("'%s.%s' function '%s' registered to '%s'",
                          type(component).__module__, type(component).__name__, func_name, queue_name)
+
+            elif str(channel).startswith(constants.INBOUND_MSG_DEST_PREFIX):
+                # If name for inbound q in app.config file, use that
+                try:
+                    app_config_q_name = component.app_configs.get(constants.INBOUND_MSG_APP_CONFIG_Q_NAME)
+                except AttributeError as e:
+                    raise IntegrationError(u"'{0}' does not have app_configs defined\n{1}".format(type(component).__name__, str(e)))
+                if app_config_q_name:
+                    handler_name = app_config_q_name
+                else:
+                    handler_name = channel.split(".", 1)[1]
+                queue_name = u"{0}.{1}.{2}".format(constants.INBOUND_MSG_DEST_PREFIX, self.org_id, handler_name)
+                LOG.info("'%s.%s' inbound handler '%s' registered to '%s'", type(component).__module__, type(component).__name__, handler_name, queue_name)
+
             else:
                 continue
 
@@ -667,7 +695,11 @@ class Actions(ResilientComponent):
         """Actually subscribe the STOMP queue.  Note: this use client-ack, not auto-ack"""
         if self.resilient_mock:
             return
-        if self.stomp_component and self.stomp_component.connected and self.listeners[queue_name]:
+
+        if queue_name.startswith(constants.INBOUND_MSG_DEST_PREFIX):
+            self.fire(Subscribe(queue_name, additional_headers=self.subscribe_headers))
+
+        elif self.stomp_component and self.stomp_component.connected and self.listeners[queue_name]:
             if queue_name in self.stomp_component.subscribed:
                 LOG.info("Ignoring request to subscribe to %s.  Already subscribed", queue_name)
             LOG.info("Subscribe to message destination '%s'", queue_name)
@@ -733,21 +765,28 @@ class Actions(ResilientComponent):
     def exception(self, etype, value, traceback, handler=None, fevent=None):
         """Report an exception thrown during handling of an action event"""
         try:
-            message = u""
+            log_message, message = u"", u""
             if etype and issubclass(etype, BaseFunctionError):
                 try:
                     message += str(value)
                 except UnicodeDecodeError:
                     message += unicode(value)
+                log_message = message
             else:
                 if etype:
-                    message = message + etype.__name__ + u": <{}>".format(value)
+                    message = u"ERROR:\n{0}\n{1}".format(message, value)
                 else:
                     message = u"Processing failed"
                 if traceback and isinstance(traceback, list):
-                    message = message + "\n" + ("".join(traceback))
 
-            LOG.error(u"%s (%s): %s", repr(fevent), repr(etype), message)
+                    str_traceback = "".join(traceback)
+
+                    if logging.getLogger().getEffectiveLevel() == logging.DEBUG:
+                        message = u"{0}\n{1}".format(message, str_traceback)
+
+                    log_message = u"{0}\n{1}".format(message, str_traceback)
+
+            LOG.error(u"%s (%s): %s", repr(fevent), repr(etype), log_message)
 
             # Try find the underlying Action or Function message
             if fevent and fevent.args and not isinstance(fevent, ActionMessageBase):
@@ -755,7 +794,18 @@ class Actions(ResilientComponent):
                     if isinstance(arg, ActionMessageBase):
                         fevent = arg
                         break
-            if fevent and isinstance(fevent, ActionMessageBase):
+
+            if isinstance(fevent, InboundMessage):
+                headers = fevent.hdr()
+                message_id = headers.get("message-id", None)
+                if not fevent.test:
+                    LOG.debug("Exception raised.\nAcknowledging InboundMessage: %s for queue: %s", message_id, headers.get("subscription", "Unknown"))
+                    self.fire(Ack(fevent.frame))
+
+            elif fevent and isinstance(fevent, ActionMessageBase):
+                # For a ActionMessageBase type
+                # 1. Ack message
+                # 2. Send a message of type 1 containing the exception text
                 fevent.stop()  # Stop further event processing
                 status = 1
                 headers = fevent.hdr()
@@ -927,8 +977,16 @@ class Actions(ResilientComponent):
         """Report the successful handling of an action event"""
         if isinstance(event.parent, ActionMessageBase) and event.name.endswith("_success"):
             fevent = event.parent
+            headers = fevent.hdr()
+            message_id = headers.get("message-id", None)
+
             if fevent.deferred:
                 LOG.debug("Not acking deferred message %s", str(fevent))
+
+            elif isinstance(fevent, InboundMessage):
+                if not fevent.test:
+                    LOG.debug("Acknowledging InboundMessage: %s for queue: %s", message_id, headers.get("subscription", "Unknown"))
+                    self.fire(Ack(fevent.frame))
             else:
                 value = event.parent.value.getValue()
                 LOG.debug("success! %s, %s", value, fevent)
@@ -1026,4 +1084,4 @@ class Actions(ResilientComponent):
                         self.fire(result, '*')
                     # Send a message back to the test client
                     self.fire(Event.create("test_response", fevent.test_msg_id, reply_message), '*')
-                    LOG.debug("Test Action: No ack done.")
+                    LOG.debug("Test Action: No ack done.")
\ No newline at end of file
